﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ParameterlessPopulationPyramid {
  [Item("Deceptive Step Trap Problem", "Genome encodes completely separable blocks, where each block deceptive with fitness plateaus.")]
  [StorableClass]
  [Creatable("Parameterless Population Pyramid")]
  // In the GECCO paper, Section 4.1
  public class DeceptiveStepTrapProblem : DeceptiveTrapProblem {
    [StorableConstructor]
    protected DeceptiveStepTrapProblem(bool deserializing) : base(deserializing) { }
    protected DeceptiveStepTrapProblem(DeceptiveStepTrapProblem original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DeceptiveStepTrapProblem(this, cloner);
    }

    private const string StepSizeParameterName = "Step Size";

    public IFixedValueParameter<IntValue> StepSizeParameter {
      get { return (IFixedValueParameter<IntValue>)Parameters[StepSizeParameterName]; }
    }

    public int StepSize {
      get { return StepSizeParameter.Value.Value; }
      set { StepSizeParameter.Value.Value = value; }
    }

    public DeceptiveStepTrapProblem()
      : base() {
      Parameters.Add(new FixedValueParameter<IntValue>(StepSizeParameterName, "", new IntValue(2)));
    }

    private int Offset {
      get { return (TrapSize - StepSize) % StepSize; }
    }

    protected override int TrapMaximum {
      get { return (Offset + TrapSize) / StepSize; }
    }

    protected override int Score(bool[] individual, int trapIndex) {
      int partial = base.Score(individual, trapIndex);
      // introduce plateaus using integer division
      return (Offset + partial) / StepSize;
    }
  }
}
